@@ -518,16 +518,9 @@ bool KFileItemDelegate::Private::isListView(const QStyleOptionViewItem &option)
 
 QPixmap KFileItemDelegate::Private::applyHoverEffect(const QPixmap &icon) const
 {
-    KIconEffect *effect = KIconLoader::global()->iconEffect();
-
-    // Note that in KIconLoader terminology, active = hover.
-    // ### We're assuming that the icon group is desktop/filemanager, since this
-    //     is KFileItemDelegate.
-    if (effect->hasEffect(KIconLoader::Desktop, KIconLoader::ActiveState)) {
-        return effect->apply(icon, KIconLoader::Desktop, KIconLoader::ActiveState);
-    }
-
-    return icon;
+    QPixmap result = icon;
+    KIconEffect::toActive(result);
+    return result;
 }
 
 void KFileItemDelegate::Private::gotNewIcon(const QModelIndex &index)
